# Memory Benchmark

The `MemoryBenchmark.ino` was compiled with each `FEATURE_*` and the flash
memory and static RAM sizes were recorded. The `FEATURE_BASELINE` selection is
the baseline, and its memory usage  numbers are subtracted from the subsequent
`FEATURE_*` memory usage.

**Version**: AceTime v1.7.4

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## How to Regenerate

To regenerate this README.md:

```
$ make clean_benchmarks
$ make benchmarks
$ make README.md
```

The `make benchmarks` target uses `collect.sh` script which calls `auniter.sh`
(https://github.com/bxparks/AUniter) to invoke the Arduino IDE programmatically.
It produces a `*.txt` file with the flash and ram usage information (e.g.
`nano.txt`). It now takes about 16 minutes to generate the `*.txt` files on my
quad-core Intel Core i7-3840QM CPU @ 2.80GHz laptop.

The `make README.md` command calls the `generated_readme.py` Python script which
generates this `README.md` file. The ASCII tables below are generated by the
`generate_table.awk` script, which takes each `*.txt` file and converts it to an
ASCII table.

## Library Size Changes

In v1.3, the `BasicZoneManager` and `ExtendedZoneManager` classes were unified
under a new parent interface `ZoneManager`. This seems to have caused the flash
size to increase by around 1200 bytes on the AVR processors (Nano, Pro Micro),
about 500 bytes on a SAMD, about 800 bytes on a ESP8266, 100 bytes on a ESP32,
and 1400 bytes on a Teensy 3.2. The 8-bit processors suffer the most
flash size increase proportional to their limited 32kB limit.

Adding the `ZoneManager` interface simplifies a lot of the complexity with
saving and restoring time zones using the `TimeZoneData` object, and I think it
is worth the extra cost of flash size. The mitigating factor is that
applications targetted towards 8-bit processors will normally have fixed number
of timezones at compile time, so they can avoid using a `ZoneManager`, and avoid
this penalty in flash size.

In v1.4.1+, we removed the `ZoneInfo::transitionBufSize` field from the
`ZoneInfo` struct, which saves 1 byte on 8-bit processors (none on 32-bit
processors due to 4-byte alignment). We save 266 bytes for `BasicZoneManager`
and 386 bytes for `ExtendedZoneManager` when all the zones are loaded into the
zone registry.

Also for v1.4.1+, incorporating zoneName compression causes flash/ram usage to
increase by ~250/120 bytes when using only 1-2 zones, but *decreases* flash
consumption by 1200-2400 bytes when all the zones are loaded into the
`ZoneManager`.

In v1.5+, changing just a single method, `ZoneProcessorCache::getType()`, from a
`virtual` to a non-virtual method saves 250-350 bytes of flash memory when using
a `BasicZoneManager` or an `ExtendedZoneManager` on an 8-bit AVR processor.
Unexpectedly, the flash memory consumption *increases* slightly (~0-50 bytes)
for some ARM processors and the ESP32. Since those processors have far more
flash memory, this seems like a good tradeoff.

Also in v1.5+, changing `BasicZoneProcessor` and `ExtendedZoneProcessor` to be
subclasses of the templatized `BasicZoneProcessorTemplate` and
`ExtendedZoneProcessorTemplate` classes causes reduction of flash consumption by
250-400 bytes for 32-bit processors. Don't know why. (Very little difference for
8-bit AVR). Adding a `BrokerFactory` layer of indirection (to support more
complex ZoneProcessors and Brokers) causes flash memory to go up by 100-200
bytes.

In v1.6, support for `LinkRegistry` was added to `BasicZoneManager` and
`ExtendedZoneManager`. This increases the flash memory usage by 150-500 bytes
when using one of these classes due to the code required by `LinkRegistrar`.
This extra cost is incurred even if the `LinkRegistry` is set to 0 elements.
Each `LinkEntry` consumes 8 bytes (2 x `uint32_t`). So a `zonedb::kLinkRegistry`
with 183 elements uses 1464 extra bytes of flash; a `zonedbx::kLinkRegistry`
with 207 elements uses 1656 extra bytes.

In v1.7, the virtual destructor on the `Clock` base class was removed. This
reduced the flash usage by 618 bytes on AVR processors , 328 bytes on the
SAMD21, but only 50-60 bytes on other 32-bit processors. The various
`printShortNameTo()` or `printShortTo()` methods were changed to replace the
underscore in the zone names (e.g. `Los_Angeles`) with spaces (e.g. `Los
Angeles`) to be more human friendly. This made little difference in the flash
memory consumption, except on the ESP32 where it increased by 200-300 bytes.

In v1.7.2, the `SystemClock::clockMillis()` became non-virtual, using
compile-time polymorphism through C++ template, and incorporating the same
techniques from AceRoutine v1.3. Saves about 20-40 bytes of flash.

## Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.13
* Arduino AVR Boards 1.8.3

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |    448/   10 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   1648/  123 |  1200/  113 |
| ZonedDateTime                          |   2440/  123 |  1992/  113 |
| Manual ZoneManager                     |   2684/  123 |  2236/  113 |
| Basic TimeZone (1 zone)                |   7330/  309 |  6882/  299 |
| Basic TimeZone (2 zones)               |   7834/  313 |  7386/  303 |
| BasicZoneManager (1 zone)              |   9056/  329 |  8608/  319 |
| BasicZoneManager (zones)               |  22022/  705 | 21574/  695 |
| BasicZoneManager (zones+thin links)    |  23558/  705 | 23110/  695 |
| BasicZoneManager (zones+fat links)     |  26114/  705 | 25666/  695 |
| Extended TimeZone (1 zone)             |  10258/  343 |  9810/  333 |
| Extended TimeZone (2 zones)            |  10900/  347 | 10452/  337 |
| ExtendedZoneManager (1 zone)           |  11952/  363 | 11504/  353 |
| ExtendedZoneManager (zones)            |  34008/  847 | 33560/  837 |
| ExtendedZoneManager (zones+thin links) |  35732/  847 | 35284/  837 |
| ExtendedZoneManager (zones+fat links)  |  38652/  847 | 38204/  837 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |   5054/  262 |  4606/  252 |
| SystemClock+Basic TimeZone             |   9816/  436 |  9368/  426 |
| SystemClock+Extended TimeZone          |  12792/  470 | 12344/  460 |
+---------------------------------------------------------------------+

```

## Sparkfun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.13
* SparkFun AVR Boards 1.1.13

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   3464/  150 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |   4772/  263 |  1308/  113 |
| ZonedDateTime                          |   5564/  263 |  2100/  113 |
| Manual ZoneManager                     |   5808/  263 |  2344/  113 |
| Basic TimeZone (1 zone)                |  10430/  447 |  6966/  297 |
| Basic TimeZone (2 zones)               |  10936/  453 |  7472/  303 |
| BasicZoneManager (1 zone)              |  12156/  467 |  8692/  317 |
| BasicZoneManager (zones)               |  25124/  845 | 21660/  695 |
| BasicZoneManager (zones+thin links)    |  26660/  845 | 23196/  695 |
| BasicZoneManager (zones+fat links)     |  29216/  845 | 25752/  695 |
| Extended TimeZone (1 zone)             |  13358/  481 |  9894/  331 |
| Extended TimeZone (2 zones)            |  14002/  487 | 10538/  337 |
| ExtendedZoneManager (1 zone)           |  15052/  501 | 11588/  351 |
| ExtendedZoneManager (zones)            |  37108/  985 | 33644/  835 |
| ExtendedZoneManager (zones+thin links) |  38832/  985 | 35368/  835 |
| ExtendedZoneManager (zones+fat links)  |  41752/  985 | 38288/  835 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |   8038/  402 |  4574/  252 |
| SystemClock+Basic TimeZone             |  12798/  574 |  9334/  424 |
| SystemClock+Extended TimeZone          |  15774/  608 | 12310/  458 |
+---------------------------------------------------------------------+

```

## SAMD21 M0 Mini

* 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.13
* Sparkfun SAMD Boards 1.8.1

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  10064/    0 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  10920/    0 |   856/    0 |
| ZonedDateTime                          |  11160/    0 |  1096/    0 |
| Manual ZoneManager                     |  11176/    0 |  1112/    0 |
| Basic TimeZone (1 zone)                |  15404/    0 |  5340/    0 |
| Basic TimeZone (2 zones)               |  15660/    0 |  5596/    0 |
| BasicZoneManager (1 zone)              |  16300/    0 |  6236/    0 |
| BasicZoneManager (zones)               |  33844/    0 | 23780/    0 |
| BasicZoneManager (zones+thin links)    |  35348/    0 | 25284/    0 |
| BasicZoneManager (zones+fat links)     |  40084/    0 | 30020/    0 |
| Extended TimeZone (1 zone)             |  17140/    0 |  7076/    0 |
| Extended TimeZone (2 zones)            |  17468/    0 |  7404/    0 |
| ExtendedZoneManager (1 zone)           |  18052/    0 |  7988/    0 |
| ExtendedZoneManager (zones)            |  48124/    0 | 38060/    0 |
| ExtendedZoneManager (zones+thin links) |  49820/    0 | 39756/    0 |
| ExtendedZoneManager (zones+fat links)  |  55212/    0 | 45148/    0 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |  12944/    0 |  2880/    0 |
| SystemClock+Basic TimeZone             |  16572/    0 |  6508/    0 |
| SystemClock+Extended TimeZone          |  18460/    0 |  8396/    0 |
+---------------------------------------------------------------------+

```

(SAMD compiler does not produce RAM usage numbers.)

## STM32 Blue Pill

* STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.13
* STM32duino 1.9.0

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |  19136/ 3788 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  23128/ 3988 |  3992/  200 |
| ZonedDateTime                          |  23280/ 3988 |  4144/  200 |
| Manual ZoneManager                     |  23288/ 3988 |  4152/  200 |
| Basic TimeZone (1 zone)                |  27200/ 3988 |  8064/  200 |
| Basic TimeZone (2 zones)               |  27472/ 3988 |  8336/  200 |
| BasicZoneManager (1 zone)              |  28052/ 3988 |  8916/  200 |
| BasicZoneManager (zones)               |  45328/ 3988 | 26192/  200 |
| BasicZoneManager (zones+thin links)    |  46824/ 3988 | 27688/  200 |
| BasicZoneManager (zones+fat links)     |  51428/ 3988 | 32292/  200 |
| Extended TimeZone (1 zone)             |  28708/ 3988 |  9572/  200 |
| Extended TimeZone (2 zones)            |  29028/ 3988 |  9892/  200 |
| ExtendedZoneManager (1 zone)           |  29632/ 3988 | 10496/  200 |
| ExtendedZoneManager (zones)            |  59288/ 3988 | 40152/  200 |
| ExtendedZoneManager (zones+thin links) |  60948/ 3988 | 41812/  200 |
| ExtendedZoneManager (zones+fat links)  |  66208/ 3988 | 47072/  200 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |  25948/ 3988 |  6812/  200 |
| SystemClock+Basic TimeZone             |  29596/ 3988 | 10460/  200 |
| SystemClock+Extended TimeZone          |  31188/ 3988 | 12052/  200 |
+---------------------------------------------------------------------+

```

An entry of `-1` indicates that the memory usage exceeded the maximum of the
microcontroller and the compiler did not generate the desired information.

## ESP8266

* NodeMCU 1.0, 80MHz ESP8266
* Arduino IDE 1.8.13
* ESP8266 Boards 2.7.4

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 256700/26776 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 259992/27300 |  3292/  524 |
| ZonedDateTime                          | 260168/27300 |  3468/  524 |
| Manual ZoneManager                     | 260248/27300 |  3548/  524 |
| Basic TimeZone (1 zone)                | 265968/27860 |  9268/ 1084 |
| Basic TimeZone (2 zones)               | 266336/27860 |  9636/ 1084 |
| BasicZoneManager (1 zone)              | 267200/27860 | 10500/ 1084 |
| BasicZoneManager (zones)               | 284816/27860 | 28116/ 1084 |
| BasicZoneManager (zones+thin links)    | 286352/27860 | 29652/ 1084 |
| BasicZoneManager (zones+fat links)     | 290928/27860 | 34228/ 1084 |
| Extended TimeZone (1 zone)             | 268064/28004 | 11364/ 1228 |
| Extended TimeZone (2 zones)            | 268416/28004 | 11716/ 1228 |
| ExtendedZoneManager (1 zone)           | 269296/28004 | 12596/ 1228 |
| ExtendedZoneManager (zones)            | 299540/28008 | 42840/ 1232 |
| ExtendedZoneManager (zones+thin links) | 301236/28008 | 44536/ 1232 |
| ExtendedZoneManager (zones+fat links)  | 306452/28008 | 49752/ 1232 |
|----------------------------------------+--------------+-------------|
| SystemClock                            | 263204/27304 |  6504/  528 |
| SystemClock+Basic TimeZone             | 268544/27860 | 11844/ 1084 |
| SystemClock+Extended TimeZone          | 270688/28004 | 13988/ 1228 |
+---------------------------------------------------------------------+

```

## ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.13
* ESP32 Boards 1.0.4

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               | 197748/13084 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          | 209950/14200 | 12202/ 1116 |
| ZonedDateTime                          | 210262/14200 | 12514/ 1116 |
| Manual ZoneManager                     | 210306/14200 | 12558/ 1116 |
| Basic TimeZone (1 zone)                | 214930/14200 | 17182/ 1116 |
| Basic TimeZone (2 zones)               | 215314/14200 | 17566/ 1116 |
| BasicZoneManager (1 zone)              | 216022/14200 | 18274/ 1116 |
| BasicZoneManager (zones)               | 233738/14200 | 35990/ 1116 |
| BasicZoneManager (zones+thin links)    | 235230/14200 | 37482/ 1116 |
| BasicZoneManager (zones+fat links)     | 240074/14200 | 42326/ 1116 |
| Extended TimeZone (1 zone)             | 216810/14200 | 19062/ 1116 |
| Extended TimeZone (2 zones)            | 217182/14200 | 19434/ 1116 |
| ExtendedZoneManager (1 zone)           | 217918/14200 | 20170/ 1116 |
| ExtendedZoneManager (zones)            | 248234/14200 | 50486/ 1116 |
| ExtendedZoneManager (zones+thin links) | 249934/14200 | 52186/ 1116 |
| ExtendedZoneManager (zones+fat links)  | 255450/14200 | 57702/ 1116 |
|----------------------------------------+--------------+-------------|
| SystemClock                            | 217790/14312 | 20042/ 1228 |
| SystemClock+Basic TimeZone             | 222042/14312 | 24294/ 1228 |
| SystemClock+Extended TimeZone          | 223950/14312 | 26202/ 1228 |
+---------------------------------------------------------------------+

```

RAM usage remains constant as more objects are created, which indicates that an
initial pool of a certain minimum size is created regardless of the actual RAM
usage by objects.

## Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.13
* Teensyduino 1.53

```
+---------------------------------------------------------------------+
| Functionality                          |  flash/  ram |       delta |
|----------------------------------------+--------------+-------------|
| Baseline                               |   7624/ 3048 |     0/    0 |
|----------------------------------------+--------------+-------------|
| LocalDateTime                          |  13268/ 4812 |  5644/ 1764 |
| ZonedDateTime                          |  13268/ 4812 |  5644/ 1764 |
| Manual ZoneManager                     |  13268/ 4812 |  5644/ 1764 |
| Basic TimeZone (1 zone)                |  22392/ 4812 | 14768/ 1764 |
| Basic TimeZone (2 zones)               |  23120/ 4812 | 15496/ 1764 |
| BasicZoneManager (1 zone)              |  24548/ 4812 | 16924/ 1764 |
| BasicZoneManager (zones)               |  42228/ 4812 | 34604/ 1764 |
| BasicZoneManager (zones+thin links)    |  43756/ 4812 | 36132/ 1764 |
| BasicZoneManager (zones+fat links)     |  48572/ 4812 | 40948/ 1764 |
| Extended TimeZone (1 zone)             |  23908/ 4812 | 16284/ 1764 |
| Extended TimeZone (2 zones)            |  24636/ 4812 | 17012/ 1764 |
| ExtendedZoneManager (1 zone)           |  26064/ 4812 | 18440/ 1764 |
| ExtendedZoneManager (zones)            |  56360/ 4812 | 48736/ 1764 |
| ExtendedZoneManager (zones+thin links) |  58080/ 4812 | 50456/ 1764 |
| ExtendedZoneManager (zones+fat links)  |  63568/ 4812 | 55944/ 1764 |
|----------------------------------------+--------------+-------------|
| SystemClock                            |  16116/ 4812 |  8492/ 1764 |
| SystemClock+Basic TimeZone             |  25428/ 4812 | 17804/ 1764 |
| SystemClock+Extended TimeZone          |  26944/ 4812 | 19320/ 1764 |
+---------------------------------------------------------------------+

```

